@@ -11,7 +11,7 @@
  *
  * Authors: Kyller Gorgonio <kyllercg@gmail.com>
  *          Marcos Morais <morais@embedded.ufcg.edu.br>
- *          Walter Guerra <wguerra@ee.ufcg.edu.br>
+ *          Walter Guerra <walter.guerra@signove.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -47,6 +47,8 @@
 /* Systems includes */
 #include <string.h>
 
+
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -54,6 +56,13 @@
 #include <nm_wlan_utils.h>
 #include <nm_bt_utils.h>
 #include <nm_settings.h>
+#include <nm_environment_manager.h>
+#include <nm_usb_setup.h>
+#include <nm_bt_setup.h>
+#include <nm_wlan_setup.h>
+#include <nm_sbrsh_setup.h>
+#include <nm_unfs_setup.h>
+#include <nm_vnc_setup.h>
 
 #include "connection-switcher-statusbar.h"
 
@@ -91,6 +100,8 @@ struct _CsStatusBarPrivate
     // This is for bluetooth search and connect
     NmBtUtils *bt_utils;
     gchar *bt_addr;
+    
+    GtkWidget *environments; /* Environments submenu */
 };
 
 static CsStatusBarPrivate *options;
@@ -105,6 +116,8 @@ static void cs_status_bar_finalize (GObject *);
 
 static void cs_set_icon (const gchar *);
 static void cs_menu_setup (void);
+static void cs_enviroment_menu_setup ();
+static void cs_environment_change(GtkWidget *, gpointer);
 
 static void cs_icon_pressed (GtkWidget *, gpointer);
 static void cs_activate_usb_storage (GtkWidget *, gpointer);
@@ -131,6 +144,7 @@ cs_status_bar_init (CsStatusBar *self)
 
     options = priv;  // Save static pointer
     ULOG_OPEN("connection-switcher-statusbar");
+    ULOG_WARN ("%s: initializing connection-switcher\n", __FUNCTION__);
 
     g_return_if_fail (priv);
 
@@ -141,7 +155,7 @@ cs_status_bar_init (CsStatusBar *self)
     priv->bt_addr = NULL;
     priv->bt_utils = nm_bt_utils_new ();
 
-    cs_set_icon ("qgn_list_connection_manager");
+    cs_set_icon ("connection-switcher");
 
     gtk_container_add (GTK_CONTAINER (priv->button), GTK_WIDGET (priv->icon));
     gtk_container_add (GTK_CONTAINER (self), priv->button);
@@ -165,6 +179,8 @@ cs_status_bar_finalize (GObject *object)
 {
     CsStatusBarPrivate *priv = CS_STATUS_BAR_GET_PRIVATE (object);
 
+    ULOG_WARN ("%s: finalizing connection-switcher\n", __FUNCTION__);
+
     g_free (priv->bt_addr);
     g_object_unref (priv->bt_utils);
 
@@ -190,12 +206,138 @@ cs_set_icon (const gchar *name)
     gtk_image_set_from_pixbuf (GTK_IMAGE (options->icon), options->pixbuf);
 }
 
+static void
+cs_enviroment_menu_setup()
+{
+        openlog("Walter", LOG_NDELAY, LOG_USER);
+        syslog(LOG_ALERT, "cs_environment_menu_setup");
+               
+        
+        if (options->environments != NULL) {
+            gtk_container_remove(GTK_CONTAINER(options->menu), options->environments );
+        }
+        
+        /* Put the environments at the menu*/
+        GtkWidget *env_menu = gtk_menu_new();
+        
+        // Envs
+        options->environments = gtk_menu_item_new_with_label ("Environments");
+        gtk_menu_item_set_submenu(GTK_MENU_ITEM(options->environments), env_menu);
+        
+        // Iterate to get all the available environments
+        GValue vt = {0,};
+        g_value_init (&vt, G_TYPE_BOOLEAN);
+        g_value_set_boolean (&vt, TRUE);
+
+        gchar *current = nm_environment_manager_get_active();
+        g_debug("[%s] - Active env: %s", __FUNCTION__, current);
+
+        /* Add the None */        
+        GtkWidget *none = gtk_check_menu_item_new_with_label(NM_ENVIRONMENT_MANAGER_NONE);
+        gtk_menu_shell_append (GTK_MENU_SHELL (env_menu),
+                                none);
+        g_signal_connect (G_OBJECT (none), "activate",
+                            G_CALLBACK (cs_environment_change), NULL); 
+                                   
+        if ( !strcmp(NM_ENVIRONMENT_MANAGER_NONE, current) ) {
+            g_object_set_property (G_OBJECT (none),
+                                    "active", &vt);
+        }
+        
+        gchar **list;
+        int i;
+        gsize length = 0;
+        
+        list = nm_environment_manager_get_list(&length);
+        syslog(LOG_ALERT, "size: %d", length);
+        GtkWidget *items[length];
+        for(i = 0; i < length; i++) {
+            items[i] = gtk_check_menu_item_new_with_label(list[i]);
+            gtk_menu_shell_append (GTK_MENU_SHELL (env_menu),
+                                    items[i]);
+            g_signal_connect (G_OBJECT (items[i]), "activate",
+                              G_CALLBACK (cs_environment_change), NULL);
+                              
+            // Mark the current one
+            if ( !strcmp(list[i], current) ) {
+                g_object_set_property (G_OBJECT (items[i]),
+                                        "active", &vt);
+            }
+        }
+        
+        // Add the submenu to the menu
+        gtk_menu_shell_prepend (GTK_MENU_SHELL (options->menu),
+                                options->environments);
+
+    
+        gtk_widget_show_all(options->environments);
+        closelog(); 
+}
+
+static void
+cs_environment_change(GtkWidget *widget, gpointer data)
+{
+    
+    gchar *clicked_env = gtk_label_get_text( GTK_LABEL(GTK_BIN(widget)->child) );
+    gchar *active_env = nm_environment_manager_get_active();
+    
+    if (strcmp(clicked_env, active_env)) {
+        gboolean error = FALSE;
+        
+        if ( !nm_usb_setup_activate_environment(clicked_env) ) {
+            g_debug("Error applying usb settings");
+            error = TRUE;
+        }
+        
+        if ( !nm_bt_setup_activate_environment(clicked_env) ) {
+            g_debug("Error applying bt settings");
+            error = TRUE;
+        }
+        
+        if ( !nm_wlan_setup_activate_environment(clicked_env) ) {
+            g_debug("Error applying wlan settings");
+            error = TRUE;
+        }
+        
+        if ( !nm_sbrsh_setup_activate_environment(clicked_env) ) {
+            g_debug("Error applying sbrsh settings");
+            error = TRUE;
+        }
+
+        if ( !nm_unfs_setup_activate_environment(clicked_env) ) {
+            g_debug("Error applying unfs settings");
+            error = TRUE;
+        }
+
+        if ( !nm_vnc_setup_activate_environment(clicked_env) ) {
+            g_debug("Error applying x11vnc settings");
+            error = TRUE;
+        }
+        
+        if (error == FALSE) {
+            nm_environment_manager_set_active(clicked_env);
+            gchar message[80];
+            sprintf(message, "Environment %s applied successfully", clicked_env);
+            g_debug("message: %s", message);
+            hildon_banner_show_information(NULL, NULL, message);
+        }
+        
+    }
+}
+
 static void
 cs_menu_setup (void)
 {
     if (!GTK_IS_MENU (options->menu)) {
 
         options->menu = gtk_menu_new ();
+        
+        cs_enviroment_menu_setup();
+                                
+        // Add the separator
+        GtkWidget *sep = gtk_separator_menu_item_new();
+        gtk_menu_shell_append (GTK_MENU_SHELL (options->menu),
+                                sep);                                                                           
 
         /* USB file storage mode */
         options->menu_usb_storage =
@@ -248,6 +390,7 @@ cs_icon_pressed (GtkWidget *widget, gpointer data)
 
     /* Create menu */
     cs_menu_setup ();
+    cs_enviroment_menu_setup();
 
     options->usb_storage = (system("/usr/sbin/nm-usb-setup status") != 0)? TRUE : FALSE;
     options->usb_netdev = (system("/usr/sbin/nm-usb-setup status") == 0)? TRUE : FALSE;
@@ -364,12 +507,11 @@ cs_activate_usb_netdev (GtkWidget *widget, gpointer data)
         if (!nm_settings_start (settings)) {
 
             hildon_banner_show_information(NULL, NULL,
-                                            "Could not read config file!\n\
-                                             Please, open network manager and cofigure it.");
+                                            "Please, open PC-Connectivity manager and configure it.");
             return;
         }
 
-        gw = nm_settings_get_value (settings, NM_USB_SETTINGS_GROUP,
+        gw = nm_settings_get_value (settings, nm_environment_manager_get_active(), NM_USB_SETTINGS_GROUP,
                                      NM_SETTINGS_GATEWAY);
 
         USB_FLAG = !USB_FLAG;
@@ -410,14 +552,13 @@ cs_activate_bt_netdev (GtkWidget *widget, gpointer data)
         if (!nm_settings_start (settings)) {
 
             hildon_banner_show_information(NULL, NULL,
-                                            "Could not read config file!\n\
-                                             Please, open network manager and cofigure it.");
+                                            "Please, open PC-Connectivity manager and configure it.");
             return;
         }
 
-        btaddr = nm_settings_get_value (settings, NM_BT_SETTINGS_GROUP,
+        btaddr = nm_settings_get_value (settings, nm_environment_manager_get_active(), NM_BT_SETTINGS_GROUP,
                                          NM_SETTINGS_MAC);
-        gw = nm_settings_get_value (settings, NM_BT_SETTINGS_GROUP,
+        gw = nm_settings_get_value (settings, nm_environment_manager_get_active(), NM_BT_SETTINGS_GROUP,
                                      NM_SETTINGS_GATEWAY);
 
         if (strcmp (btaddr, NM_BT_DEFAULT_MAC) == 0) {
